import os
import shutil
import tempfile
import unittest
from zope.component import queryUtility
from zope.interface.verify import verifyObject, verifyClass
from waeup.sirp.interfaces import ICSVExporter
from waeup.sirp.testing import SIRPUnitTestLayer
from waeup.sirp.university import FacultiesContainer, Faculty
from waeup.sirp.university.export import FacultyExporter

class FacultyExporterTest(unittest.TestCase):

    layer = SIRPUnitTestLayer

    def setUp(self):
        self.workdir = tempfile.mkdtemp()
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        return

    def tearDown(self):
        shutil.rmtree(self.workdir)
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = FacultyExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, FacultyExporter)
        return

    def test_get_as_utility(self):
        # we can get a faculty exporter as utility
        result = queryUtility(ICSVExporter, name="Faculty Exporter")
        self.assertTrue(result is not None)
        return

    def test_export(self):
        # we can export single faculties (for what it's worth)
        fac = Faculty('Faculty of Cheese', 'faculty', 'F1')
        exporter = FacultyExporter()
        exporter.export(fac, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            )
        return

    def test_export_to_string(self):
        # we can export single faculties to a string.
        fac = Faculty('Faculty of Cheese', 'faculty', 'F1')
        exporter = FacultyExporter()
        result = exporter.export(fac, filepath=None)
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            )
        return

    def test_export_all(self):
        # we can export all faculties in a site
        container = FacultiesContainer()
        site = {'faculties':container}
        fac1 = Faculty('Faculty of Cheese', 'faculty', 'F1')
        fac2 = Faculty('Centre of Onion', 'centre', 'F2')
        container.addFaculty(fac1)
        container.addFaculty(fac2)
        exporter = FacultyExporter()
        exporter.export_all(site, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            'F2,Centre of Onion,centre\r\n'
            )
        return

    def test_export_all_to_string(self):
        # we can export all faculties in a site to a string
        container = FacultiesContainer()
        site = {'faculties':container}
        fac1 = Faculty('Faculty of Cheese', 'faculty', 'F1')
        fac2 = Faculty('Centre of Onion', 'centre', 'F2')
        container.addFaculty(fac1)
        container.addFaculty(fac2)
        exporter = FacultyExporter()
        result = exporter.export_all(site, filepath=None)
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            'F2,Centre of Onion,centre\r\n'
            )
        return
